@@ -1,27 +1,29 @@
 #!/usr/bin/env python
 
 import asyncio
+import gc
 import ipaddress
 import logging
-import unittest
-import pytest
 import socket
 import sys
 import time
+import unittest
 import unittest.mock
 from typing import Any
 
 import pycares
+import pytest
 
 import aiodns
-import pycares
 
 try:
-    if sys.platform == "win32":
+    if sys.platform == 'win32':
         import winloop as uvloop
+
         skip_uvloop = False
     else:
-        import uvloop 
+        import uvloop
+
         skip_uvloop = False
 except ModuleNotFoundError:
     skip_uvloop = True
@@ -30,22 +32,26 @@ except ModuleNotFoundError:
 class DNSTest(unittest.TestCase):
     def setUp(self) -> None:
         if sys.platform == 'win32':
-            asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
+            asyncio.set_event_loop_policy(
+                asyncio.WindowsSelectorEventLoopPolicy()
+            )
         self.loop = asyncio.new_event_loop()
         self.addCleanup(self.loop.close)
         self.resolver = aiodns.DNSResolver(loop=self.loop, timeout=5.0)
         self.resolver.nameservers = ['8.8.8.8']
 
     def tearDown(self) -> None:
+        self.loop.run_until_complete(self.resolver.close())
         self.resolver = None  # type: ignore[assignment]
 
     def test_query_a(self) -> None:
         f = self.resolver.query('google.com', 'A')
-        result = self.loop.run_until_complete(f)
+        self.loop.run_until_complete(f)
 
     def test_query_async_await(self) -> None:
         async def f() -> list[pycares.ares_query_a_result]:
             return await self.resolver.query('google.com', 'A')
+
         result = self.loop.run_until_complete(f())
         self.assertTrue(result)
 
@@ -76,7 +82,9 @@ class DNSTest(unittest.TestCase):
         result = self.loop.run_until_complete(f)
         self.assertTrue(result)
 
-    @unittest.skipIf(sys.platform == 'darwin', 'skipped on Darwin as it is flakey on CI')
+    @unittest.skipIf(
+        sys.platform == 'darwin', 'skipped on Darwin as it is flakey on CI'
+    )
     def test_query_txt(self) -> None:
         f = self.resolver.query('google.com', 'TXT')
         result = self.loop.run_until_complete(f)
@@ -99,34 +107,19 @@ class DNSTest(unittest.TestCase):
 
     def test_query_ptr(self) -> None:
         ip = '172.253.122.26'
-        f = self.resolver.query(ipaddress.ip_address(ip).reverse_pointer, 'PTR')
+        f = self.resolver.query(
+            ipaddress.ip_address(ip).reverse_pointer, 'PTR'
+        )
         result = self.loop.run_until_complete(f)
         self.assertTrue(result)
 
     def test_query_bad_type(self) -> None:
         self.assertRaises(ValueError, self.resolver.query, 'google.com', 'XXX')
 
-    def test_query_txt_chaos(self) -> None:
-        self.resolver = aiodns.DNSResolver(loop=self.loop)
-        self.resolver.nameservers = ['1.1.1.1']
-        f = self.resolver.query('id.server', 'TXT', 'CHAOS')
-        result = self.loop.run_until_complete(f)
-        self.assertTrue(result)
-
     def test_query_bad_class(self) -> None:
-        self.assertRaises(ValueError, self.resolver.query, 'google.com', 'A', "INVALIDCLASS")
-
-    def test_query_timeout(self) -> None:
-        self.resolver = aiodns.DNSResolver(timeout=0.1, tries=1, loop=self.loop)
-        self.resolver.nameservers = ['1.2.3.4']
-        f = self.resolver.query('google.com', 'A')
-        started = time.monotonic()
-        try:
-            self.loop.run_until_complete(f)
-        except aiodns.error.DNSError as e:
-            self.assertEqual(e.args[0], aiodns.error.ARES_ETIMEOUT)
-        # Ensure timeout really cuts time deadline. Limit duration to one second
-        self.assertLess(time.monotonic() - started, 1)
+        self.assertRaises(
+            ValueError, self.resolver.query, 'google.com', 'A', 'INVALIDCLASS'
+        )
 
     def test_query_cancel(self) -> None:
         f = self.resolver.query('google.com', 'A')
@@ -139,9 +132,11 @@ class DNSTest(unittest.TestCase):
     def test_future_cancel(self) -> None:
         f = self.resolver.query('google.com', 'A')
         f.cancel()
+
         async def coro() -> None:
             await asyncio.sleep(0.1)
             await f
+
         try:
             self.loop.run_until_complete(coro())
         except asyncio.CancelledError as e:
@@ -149,9 +144,10 @@ class DNSTest(unittest.TestCase):
 
     def test_query_twice(self) -> None:
         async def coro(self: DNSTest) -> None:
-            for i in range(2):
-                result = await self.resolver.query("gmail.com", "MX")
+            for _ in range(2):
+                result = await self.resolver.query('gmail.com', 'MX')
                 self.assertTrue(result)
+
         self.loop.run_until_complete(coro(self))
 
     def test_gethostbyname(self) -> None:
@@ -169,13 +165,17 @@ class DNSTest(unittest.TestCase):
         f = self.resolver.getaddrinfo('google.com', socket.AF_INET)
         result = self.loop.run_until_complete(f)
         self.assertTrue(result)
-        self.assertTrue(all(node.family == socket.AF_INET for node in result.nodes))
+        self.assertTrue(
+            all(node.family == socket.AF_INET for node in result.nodes)
+        )
 
     def test_getaddrinfo_address_family_af_inet6(self) -> None:
         f = self.resolver.getaddrinfo('google.com', socket.AF_INET6)
         result = self.loop.run_until_complete(f)
         self.assertTrue(result)
-        self.assertTrue(all(node.family == socket.AF_INET6 for node in result.nodes))
+        self.assertTrue(
+            all(node.family == socket.AF_INET6 for node in result.nodes)
+        )
 
     def test_getnameinfo_ipv4(self) -> None:
         f = self.resolver.getnameinfo(('127.0.0.1', 0))
@@ -205,11 +205,63 @@ class DNSTest(unittest.TestCase):
         with self.assertRaises(aiodns.error.DNSError):
             self.loop.run_until_complete(f)
 
+
 #    def test_query_bad_chars(self) -> None:
 #        f = self.resolver.query('xn--cardeosapeluqueros-r0b.com', 'MX')
 #        result = self.loop.run_until_complete(f)
 #        self.assertTrue(result)
 
+
+class TestQueryTxtChaos(DNSTest):
+    """Test DNS queries with CHAOS class."""
+
+    def setUp(self) -> None:
+        if sys.platform == 'win32':
+            asyncio.set_event_loop_policy(
+                asyncio.WindowsSelectorEventLoopPolicy()
+            )
+        self.loop = asyncio.new_event_loop()
+        self.addCleanup(self.loop.close)
+        self.resolver = aiodns.DNSResolver(loop=self.loop)
+        self.resolver.nameservers = ['1.1.1.1']
+
+    def test_query_txt_chaos(self) -> None:
+        f = self.resolver.query('id.server', 'TXT', 'CHAOS')
+        result = self.loop.run_until_complete(f)
+        self.assertTrue(result)
+
+
+class TestQueryTimeout(unittest.TestCase):
+    """Test DNS queries with timeout configuration."""
+
+    def setUp(self) -> None:
+        if sys.platform == 'win32':
+            asyncio.set_event_loop_policy(
+                asyncio.WindowsSelectorEventLoopPolicy()
+            )
+        self.loop = asyncio.new_event_loop()
+        self.addCleanup(self.loop.close)
+        self.resolver = aiodns.DNSResolver(
+            timeout=0.1, tries=1, loop=self.loop
+        )
+        self.resolver.nameservers = ['1.2.3.4']
+
+    def tearDown(self) -> None:
+        self.loop.run_until_complete(self.resolver.close())
+        self.resolver = None  # type: ignore[assignment]
+
+    def test_query_timeout(self) -> None:
+        f = self.resolver.query('google.com', 'A')
+        started = time.monotonic()
+        try:
+            self.loop.run_until_complete(f)
+        except aiodns.error.DNSError as e:
+            self.assertEqual(e.args[0], aiodns.error.ARES_ETIMEOUT)
+        # Ensure timeout really cuts time deadline.
+        # Limit duration to one second
+        self.assertLess(time.monotonic() - started, 1)
+
+
 @unittest.skipIf(skip_uvloop, "We don't have a uvloop or winloop module")
 class TestUV_DNS(DNSTest):
     def setUp(self) -> None:
@@ -224,7 +276,55 @@ class TestNoEventThreadDNS(DNSTest):
     """Test DNSResolver with no event thread."""
 
     def setUp(self) -> None:
-        with unittest.mock.patch('aiodns.pycares.ares_threadsafety', return_value=False):
+        with unittest.mock.patch(
+            'aiodns.pycares.ares_threadsafety', return_value=False
+        ):
+            super().setUp()
+
+
+@unittest.skipIf(skip_uvloop, "We don't have a uvloop or winloop module")
+class TestUV_QueryTxtChaos(TestQueryTxtChaos):
+    """Test DNS queries with CHAOS class using uvloop."""
+
+    def setUp(self) -> None:
+        asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
+        self.loop = asyncio.new_event_loop()
+        self.addCleanup(self.loop.close)
+        self.resolver = aiodns.DNSResolver(loop=self.loop)
+        self.resolver.nameservers = ['1.1.1.1']
+
+
+@unittest.skipIf(skip_uvloop, "We don't have a uvloop or winloop module")
+class TestUV_QueryTimeout(TestQueryTimeout):
+    """Test DNS queries with timeout configuration using uvloop."""
+
+    def setUp(self) -> None:
+        asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
+        self.loop = asyncio.new_event_loop()
+        self.addCleanup(self.loop.close)
+        self.resolver = aiodns.DNSResolver(
+            timeout=0.1, tries=1, loop=self.loop
+        )
+        self.resolver.nameservers = ['1.2.3.4']
+
+
+class TestNoEventThreadQueryTxtChaos(TestQueryTxtChaos):
+    """Test DNS queries with CHAOS class without event thread."""
+
+    def setUp(self) -> None:
+        with unittest.mock.patch(
+            'aiodns.pycares.ares_threadsafety', return_value=False
+        ):
+            super().setUp()
+
+
+class TestNoEventThreadQueryTimeout(TestQueryTimeout):
+    """Test DNS queries with timeout configuration without event thread."""
+
+    def setUp(self) -> None:
+        with unittest.mock.patch(
+            'aiodns.pycares.ares_threadsafety', return_value=False
+        ):
             super().setUp()
 
 
@@ -239,50 +339,52 @@ def test_win32_no_selector_event_loop() -> None:
 
     with (
         pytest.raises(
-            RuntimeError, match="aiodns needs a SelectorEventLoop on Windows"
+            RuntimeError, match='aiodns needs a SelectorEventLoop on Windows'
         ),
-        unittest.mock.patch("aiodns.pycares.ares_threadsafety", return_value=False),
-        unittest.mock.patch("sys.platform", "win32"),
+        unittest.mock.patch(
+            'aiodns.pycares.ares_threadsafety', return_value=False
+        ),
+        unittest.mock.patch('sys.platform', 'win32'),
     ):
         aiodns.DNSResolver(loop=mock_loop, timeout=5.0)
 
 
 @pytest.mark.parametrize(
-    ("platform", "expected_msg_parts", "unexpected_msg_parts"),
+    ('platform', 'expected_msg_parts', 'unexpected_msg_parts'),
     [
         (
-            "linux",
+            'linux',
             [
-                "automatic monitoring of",
-                "resolver configuration changes",
-                "system ran out of inotify watches",
-                "Falling back to socket state callback",
-                "Consider increasing the system inotify watch limit",
+                'automatic monitoring of',
+                'resolver configuration changes',
+                'system ran out of inotify watches',
+                'Falling back to socket state callback',
+                'Consider increasing the system inotify watch limit',
             ],
             [],
         ),
         (
-            "darwin",
+            'darwin',
             [
-                "automatic monitoring",
-                "resolver configuration changes",
-                "Falling back to socket state callback",
+                'automatic monitoring',
+                'resolver configuration changes',
+                'Falling back to socket state callback',
             ],
             [
-                "system ran out of inotify watches",
-                "Consider increasing the system inotify watch limit",
+                'system ran out of inotify watches',
+                'Consider increasing the system inotify watch limit',
             ],
         ),
         (
-            "win32",
+            'win32',
             [
-                "automatic monitoring",
-                "resolver configuration changes",
-                "Falling back to socket state callback",
+                'automatic monitoring',
+                'resolver configuration changes',
+                'Falling back to socket state callback',
             ],
             [
-                "system ran out of inotify watches",
-                "Consider increasing the system inotify watch limit",
+                'system ran out of inotify watches',
+                'Consider increasing the system inotify watch limit',
             ],
         ),
     ],
@@ -297,23 +399,27 @@ async def test_make_channel_ares_error(
     # Set log level to capture warnings
     caplog.set_level(logging.WARNING)
 
-    # Create a mock loop that is a SelectorEventLoop to avoid Windows-specific errors
+    # Create a mock loop that is a SelectorEventLoop to
+    # avoid Windows-specific errors
     mock_loop = unittest.mock.MagicMock(spec=asyncio.SelectorEventLoop)
     mock_channel = unittest.mock.MagicMock()
 
     with (
-        unittest.mock.patch("sys.platform", platform),
-        # Configure first Channel call to raise AresError, second call to return our mock
+        unittest.mock.patch('sys.platform', platform),
+        # Configure first Channel call to raise AresError,
+        # second call to return our mock
         unittest.mock.patch(
-            "aiodns.pycares.Channel",
+            'aiodns.pycares.Channel',
             side_effect=[
-                pycares.AresError("Mock error"),
+                pycares.AresError('Mock error'),
                 mock_channel,
             ],
         ),
-        unittest.mock.patch("aiodns.pycares.ares_threadsafety", return_value=True),
+        unittest.mock.patch(
+            'aiodns.pycares.ares_threadsafety', return_value=True
+        ),
         # Also patch asyncio.get_event_loop to return our mock loop
-        unittest.mock.patch("asyncio.get_event_loop", return_value=mock_loop),
+        unittest.mock.patch('asyncio.get_event_loop', return_value=mock_loop),
     ):
         # Create resolver which will call _make_channel
         resolver = aiodns.DNSResolver(loop=mock_loop)
@@ -329,9 +435,17 @@ async def test_make_channel_ares_error(
         for part in unexpected_msg_parts:
             assert part not in caplog.text
 
+        # Manually set _closed to True to prevent cleanup logic from
+        # running during the test.
+        resolver._closed = True
+
 
 def test_win32_import_winloop_error() -> None:
-    """Test handling of ModuleNotFoundError when importing winloop on Windows."""
+    """Test winloop import error on Windows.
+
+    Test handling of ModuleNotFoundError when importing
+    winloop on Windows.
+    """
     # Create a mock event loop that is not a SelectorEventLoop
     mock_loop = unittest.mock.MagicMock(spec=asyncio.AbstractEventLoop)
 
@@ -339,7 +453,7 @@ def test_win32_import_winloop_error() -> None:
     original_import = __import__
 
     def mock_import(name: str, *args: Any, **kwargs: Any) -> Any:
-        if name == "winloop":
+        if name == 'winloop':
             raise ModuleNotFoundError("No module named 'winloop'")
         return original_import(name, *args, **kwargs)
 
@@ -347,19 +461,29 @@ def test_win32_import_winloop_error() -> None:
     mock_channel = unittest.mock.MagicMock()
 
     with (
-        unittest.mock.patch("sys.platform", "win32"),
-        unittest.mock.patch("aiodns.pycares.ares_threadsafety", return_value=False),
-        unittest.mock.patch("builtins.__import__", side_effect=mock_import),
-        unittest.mock.patch("importlib.import_module", side_effect=mock_import),
+        unittest.mock.patch('sys.platform', 'win32'),
+        unittest.mock.patch(
+            'aiodns.pycares.ares_threadsafety', return_value=False
+        ),
+        unittest.mock.patch('builtins.__import__', side_effect=mock_import),
+        unittest.mock.patch(
+            'importlib.import_module', side_effect=mock_import
+        ),
         # Also patch Channel creation to avoid socket resource leak
-        unittest.mock.patch("aiodns.pycares.Channel", return_value=mock_channel),
+        unittest.mock.patch(
+            'aiodns.pycares.Channel', return_value=mock_channel
+        ),
         pytest.raises(RuntimeError, match=aiodns.WINDOWS_SELECTOR_ERR_MSG),
     ):
         aiodns.DNSResolver(loop=mock_loop)
 
 
 def test_win32_winloop_not_loop_instance() -> None:
-    """Test handling of a loop that is not a winloop.Loop instance on Windows."""
+    """Test non-winloop.Loop instance on Windows.
+
+    Test handling of a loop that is not a winloop.Loop
+    instance on Windows.
+    """
     # Create a mock event loop that is not a SelectorEventLoop
     mock_loop = unittest.mock.MagicMock(spec=asyncio.AbstractEventLoop)
 
@@ -373,7 +497,7 @@ def test_win32_winloop_not_loop_instance() -> None:
     mock_winloop_module.Loop = MockLoop
 
     def mock_import(name: str, *args: Any, **kwargs: Any) -> Any:
-        if name == "winloop":
+        if name == 'winloop':
             return mock_winloop_module
         return original_import(name, *args, **kwargs)
 
@@ -381,19 +505,28 @@ def test_win32_winloop_not_loop_instance() -> None:
     mock_channel = unittest.mock.MagicMock()
 
     with (
-        unittest.mock.patch("sys.platform", "win32"),
-        unittest.mock.patch("aiodns.pycares.ares_threadsafety", return_value=False),
-        unittest.mock.patch("builtins.__import__", side_effect=mock_import),
-        unittest.mock.patch("importlib.import_module", side_effect=mock_import),
+        unittest.mock.patch('sys.platform', 'win32'),
+        unittest.mock.patch(
+            'aiodns.pycares.ares_threadsafety', return_value=False
+        ),
+        unittest.mock.patch('builtins.__import__', side_effect=mock_import),
+        unittest.mock.patch(
+            'importlib.import_module', side_effect=mock_import
+        ),
         # Also patch Channel creation to avoid socket resource leak
-        unittest.mock.patch("aiodns.pycares.Channel", return_value=mock_channel),
+        unittest.mock.patch(
+            'aiodns.pycares.Channel', return_value=mock_channel
+        ),
         pytest.raises(RuntimeError, match=aiodns.WINDOWS_SELECTOR_ERR_MSG),
     ):
         aiodns.DNSResolver(loop=mock_loop)
 
 
 def test_win32_winloop_loop_instance() -> None:
-    """Test handling of a loop that IS a winloop.Loop instance on Windows."""
+    """Test winloop.Loop instance on Windows.
+
+    Test handling of a loop that IS a winloop.Loop instance on Windows.
+    """
 
     # Create a mock winloop module with a Loop class
     class MockLoop:
@@ -410,7 +543,7 @@ def test_win32_winloop_loop_instance() -> None:
     original_import = __import__
 
     def mock_import(name: str, *args: Any, **kwargs: Any) -> Any:
-        if name == "winloop":
+        if name == 'winloop':
             return mock_winloop_module
         return original_import(name, *args, **kwargs)
 
@@ -418,15 +551,255 @@ def test_win32_winloop_loop_instance() -> None:
     mock_channel = unittest.mock.MagicMock()
 
     with (
-        unittest.mock.patch("sys.platform", "win32"),
-        unittest.mock.patch("aiodns.pycares.ares_threadsafety", return_value=False),
-        unittest.mock.patch("builtins.__import__", side_effect=mock_import),
-        unittest.mock.patch("importlib.import_module", side_effect=mock_import),
-        unittest.mock.patch("aiodns.pycares.Channel", return_value=mock_channel),
+        unittest.mock.patch('sys.platform', 'win32'),
+        unittest.mock.patch(
+            'aiodns.pycares.ares_threadsafety', return_value=False
+        ),
+        unittest.mock.patch('builtins.__import__', side_effect=mock_import),
+        unittest.mock.patch(
+            'importlib.import_module', side_effect=mock_import
+        ),
+        unittest.mock.patch(
+            'aiodns.pycares.Channel', return_value=mock_channel
+        ),
     ):
-        # This should not raise an exception since loop is a winloop.Loop instance
+        # This should not raise an exception since loop
+        # is a winloop.Loop instance
         aiodns.DNSResolver(loop=mock_loop)
 
 
-if __name__ == "__main__":  # pragma: no cover
+@pytest.mark.asyncio
+async def test_close_resolver() -> None:
+    """Test that DNSResolver.close() properly shuts down the resolver."""
+    resolver = aiodns.DNSResolver()
+
+    # Create a query to ensure resolver is active
+    query_future = resolver.query('google.com', 'A')
+
+    # Close the resolver
+    await resolver.close()
+
+    # Verify resolver is marked as closed
+    assert resolver._closed
+
+    # Verify timers are cancelled
+    assert resolver._timer is None
+
+    # Verify file descriptors are cleared
+    assert len(resolver._read_fds) == 0
+    assert len(resolver._write_fds) == 0
+
+    # The query should fail with cancellation
+    with pytest.raises(aiodns.error.DNSError) as exc_info:
+        await query_future
+    assert exc_info.value.args[0] == aiodns.error.ARES_ECANCELLED
+
+
+@pytest.mark.asyncio
+async def test_close_resolver_multiple_times() -> None:
+    """Test that close() is idempotent and safe to call multiple times."""
+    resolver = aiodns.DNSResolver()
+
+    # Close multiple times
+    await resolver.close()
+    await resolver.close()
+    await resolver.close()
+
+    # All closes should succeed without error
+    assert resolver._closed
+
+
+def test_del_with_no_running_loop() -> None:
+    """Test __del__ when there's no running event loop."""
+    loop = asyncio.new_event_loop()
+    resolver = aiodns.DNSResolver(loop=loop)
+
+    # Track if cleanup was called via channel.close
+    cleanup_called = False
+    original_close = resolver._channel.close
+
+    def mock_close() -> None:
+        nonlocal cleanup_called
+        cleanup_called = True
+        original_close()
+
+    resolver._channel.close = mock_close  # type: ignore[method-assign]
+    loop.close()
+
+    # Delete the resolver without closing it
+    del resolver
+    gc.collect()
+
+    # Should have called cleanup
+    assert cleanup_called
+
+
+def test_del_with_stopped_event_loop() -> None:
+    """Test __del__ when event loop is not running."""
+    # Create a new event loop
+    loop = asyncio.new_event_loop()
+
+    # Create resolver with this loop
+    resolver = aiodns.DNSResolver(loop=loop)
+
+    # Track if cleanup was called via channel.close
+    cleanup_called = False
+    original_close = resolver._channel.close
+
+    def mock_close() -> None:
+        nonlocal cleanup_called
+        cleanup_called = True
+        original_close()
+
+    resolver._channel.close = mock_close  # type: ignore[method-assign]
+
+    # Close the loop so it's not running
+    loop.close()
+
+    # Delete resolver when its loop is not running
+    del resolver
+    gc.collect()
+
+    # Should have called cleanup
+    assert cleanup_called
+
+
+@pytest.mark.asyncio
+async def test_del_with_running_event_loop() -> None:
+    """Test __del__ when event loop is running performs cleanup."""
+    resolver = aiodns.DNSResolver()
+
+    # Mark that cleanup was called by checking if channel.close was called
+    original_close = resolver._channel.close
+    cleanup_called = False
+
+    def mock_close() -> None:
+        nonlocal cleanup_called
+        cleanup_called = True
+        original_close()
+
+    resolver._channel.close = mock_close  # type: ignore[method-assign]
+
+    # Delete resolver while loop is running
+    del resolver
+    gc.collect()
+
+    # Verify cleanup was called
+    assert cleanup_called
+
+
+@pytest.mark.asyncio
+async def test_cleanup_method() -> None:
+    """Test that _cleanup() properly cleans up resources."""
+    resolver = aiodns.DNSResolver()
+
+    # Mock file descriptors and timer
+    resolver._read_fds.add(1)
+    resolver._read_fds.add(2)
+    resolver._write_fds.add(3)
+    resolver._write_fds.add(4)
+
+    # Mock timer
+    mock_timer = unittest.mock.MagicMock()
+    resolver._timer = mock_timer
+
+    # Mock loop methods
+    resolver.loop.remove_reader = unittest.mock.MagicMock()  # type: ignore[method-assign]
+    resolver.loop.remove_writer = unittest.mock.MagicMock()  # type: ignore[method-assign]
+
+    # Call cleanup
+    resolver._cleanup()
+
+    # Verify timer was cancelled
+    mock_timer.cancel.assert_called_once()
+    assert resolver._timer is None
+
+    # Verify file descriptors were removed
+    resolver.loop.remove_reader.assert_any_call(1)  # type: ignore[unreachable]
+    resolver.loop.remove_reader.assert_any_call(2)
+    resolver.loop.remove_writer.assert_any_call(3)
+    resolver.loop.remove_writer.assert_any_call(4)
+
+    # Verify sets are cleared
+    assert len(resolver._read_fds) == 0
+    assert len(resolver._write_fds) == 0
+
+
+@pytest.mark.asyncio
+async def test_context_manager() -> None:
+    """Test DNSResolver as async context manager."""
+    resolver_closed = False
+
+    # Create resolver and use as context manager
+    async with aiodns.DNSResolver() as resolver:
+        # Check resolver is not closed
+        assert not resolver._closed
+
+        # Mock the close method to track if it's called
+        original_close = resolver.close
+
+        async def mock_close() -> None:
+            nonlocal resolver_closed
+            resolver_closed = True
+            await original_close()
+
+        resolver.close = mock_close  # type: ignore[method-assign]
+
+        # Resolver should be usable within context
+        assert isinstance(resolver, aiodns.DNSResolver)
+
+    # After exiting context, close should have been called
+    assert resolver_closed
+
+
+@pytest.mark.asyncio
+async def test_context_manager_with_exception() -> None:
+    """Test DNSResolver context manager handles exceptions properly."""
+    resolver_closed = False
+
+    try:
+        async with aiodns.DNSResolver() as resolver:
+            # Mock the close method to track if it's called
+            original_close = resolver.close
+
+            async def mock_close() -> None:
+                nonlocal resolver_closed
+                resolver_closed = True
+                await original_close()
+
+            resolver.close = mock_close  # type: ignore[method-assign]
+
+            # Raise an exception within the context
+            raise ValueError('Test exception')
+    except ValueError:
+        pass  # Expected
+
+    # Close should still be called even with exception
+    assert resolver_closed
+
+
+@pytest.mark.asyncio
+async def test_context_manager_close_idempotent() -> None:
+    """Test that close() can be called multiple times safely."""
+    close_count = 0
+
+    async with aiodns.DNSResolver() as resolver:
+        original_close = resolver.close
+
+        async def mock_close() -> None:
+            nonlocal close_count
+            close_count += 1
+            await original_close()
+
+        resolver.close = mock_close  # type: ignore[method-assign]
+
+        # Manually close resolver within context
+        await resolver.close()
+        assert close_count == 1
+
+    # Context manager should call close again, but it should be idempotent
+    assert close_count == 2
+
+
+if __name__ == '__main__':  # pragma: no cover
     unittest.main(verbosity=2)
